==> brainlog-1.1.1.tgz: package/src/main.cc <==
#include <node.h>
#include <iostream>
#include "Matrix.h"
#include "Vector.h"
#include "LogisticMFEngine.h"
#include "ALSEngine.h"
#include "Metrics.h"

namespace demo {

using v8::FunctionCallbackInfo;
using v8::Isolate;
using v8::Local;
using v8::Object;
using v8::String;
using v8::Value;
using v8::Array;

void testLogisticMf(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  Matrix userVectors(Local<Array>::Cast(args[1]));
  Matrix itemVectors(Local<Array>::Cast(args[2]));
  Vector userBiases(Local<Array>::Cast(args[3]));
  Vector itemBiases(Local<Array>::Cast(args[4]));

  double regularization = args[5]->NumberValue();
  double gamma = args[6]->NumberValue();
  size_t iterations = args[7]->IntegerValue();
  size_t factors = args[8]->IntegerValue();

  size_t numUsers = ratingMatrix.rows();
  size_t numItems = ratingMatrix.cols();
  size_t numZeros = numUsers * numItems;
  double total = 0;

  for(size_t i = 0; i < numUsers; i++) {
    for(size_t j = 0; j < numItems; j++) {
      double value = ratingMatrix(i, j);
      if(value == 0) continue;
      numZeros -= 1;
      total += value;
    }
  }

  double alpha = numZeros / total;
  Matrix newRatingMatrix = ratingMatrix * alpha;

  LogisticMFEngine engine(
    newRatingMatrix,
    userVectors, itemVectors,
    userBiases, itemBiases,
    regularization, gamma,
    iterations, factors
  );

  Matrix X = engine.train();

  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void logisticMF(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  double regularization = args[1]->NumberValue();
  double gamma = args[2]->NumberValue();
  size_t factors = args[3]->IntegerValue();
  size_t iterations = args[4]->IntegerValue();
  size_t seed = args[5]->IntegerValue();

  size_t numUsers = ratingMatrix.rows();
  size_t numItems = ratingMatrix.cols();
  size_t numZeros = numUsers * numItems;
  double total = 0;

  for(size_t i = 0; i < numUsers; i++) {
    for(size_t j = 0; j < numItems; j++) {
      double value = ratingMatrix(i, j);
      if(value == 0) continue;
      numZeros -= 1;
      total += value;
    }
  }

  double alpha = numZeros / total;
  Matrix newRatingMatrix = ratingMatrix * alpha;

  LogisticMFEngine engine(
    newRatingMatrix,
    factors, iterations,
    regularization, gamma,
    seed
  );

  Matrix X = engine.train();

  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void testAls(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  Matrix userFactors(Local<Array>::Cast(args[1]));
  Matrix itemFactors(Local<Array>::Cast(args[2]));

  double regularization = args[3]->NumberValue();
  size_t iterations = args[4]->IntegerValue();

  ALSEngine engine(
    ratingMatrix,
    userFactors,
    itemFactors,
    regularization,
    iterations
  );

  Matrix X = engine.train();
  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void als(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  double alpha = args[1]->NumberValue();
  size_t factors = args[2]->IntegerValue();
  double regularization = args[3]->NumberValue();
  size_t iterations = args[4]->IntegerValue();
  size_t seed = args[5]->IntegerValue();

  ALSEngine engine(
    ratingMatrix,
    alpha,
    regularization,
    factors,
    iterations,
    seed
  );

  Matrix X = engine.train();
  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void auc(const FunctionCallbackInfo<Value>& args) {

  Local<Array> labels = Local<Array>::Cast(args[0]);
  Local<Array> scores = Local<Array>::Cast(args[1]);

  size_t length = labels->Length();

  std::vector<double> newLabels;
  std::vector<double> newScores;

  for(size_t i = 0; i < length; i++) {
    newLabels.push_back( labels->Get(i)->NumberValue() );
    newScores.push_back( scores->Get(i)->NumberValue() );
  }

  AUC aucMetric;
  double aucScore = aucMetric.compute(newLabels, newScores);

  args.GetReturnValue().Set(aucScore);
}

void init(Local<Object> exports) {

  // Metrics
  NODE_SET_METHOD(exports, "auc", auc);

  // Production Algorithms
  NODE_SET_METHOD(exports, "logisticMF", logisticMF);
  NODE_SET_METHOD(exports, "als", als);

  // Testing Algorithms
  NODE_SET_METHOD(exports, "testLogisticMf", testLogisticMf);
  NODE_SET_METHOD(exports, "testAls", testAls);
}

NODE_MODULE(main, init)

}  // namespace demo

==> brainlog-1.3.0.tgz: package/src/main.cc <==
#include <node.h>
#include <iostream>
#include <vector>
#include "Matrix.h"
#include "Vector.h"
#include "LogisticMFEngine.h"
#include "ALSEngine.h"
#include "BPREngine.h"
#include "Metrics.h"

namespace demo {

using v8::FunctionCallbackInfo;
using v8::Isolate;
using v8::Local;
using v8::Object;
using v8::String;
using v8::Value;
using v8::Array;

void testLogisticMf(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  Matrix userVectors(Local<Array>::Cast(args[1]));
  Matrix itemVectors(Local<Array>::Cast(args[2]));
  Vector userBiases(Local<Array>::Cast(args[3]));
  Vector itemBiases(Local<Array>::Cast(args[4]));

  double regularization = args[5]->NumberValue();
  double gamma = args[6]->NumberValue();
  size_t iterations = args[7]->IntegerValue();
  size_t factors = args[8]->IntegerValue();

  size_t numUsers = ratingMatrix.rows();
  size_t numItems = ratingMatrix.cols();
  size_t numZeros = numUsers * numItems;
  double total = 0;

  for(size_t i = 0; i < numUsers; i++) {
    for(size_t j = 0; j < numItems; j++) {
      double value = ratingMatrix(i, j);
      if(value == 0) continue;
      numZeros -= 1;
      total += value;
    }
  }

  double alpha = numZeros / total;
  Matrix newRatingMatrix = ratingMatrix * alpha;

  LogisticMFEngine engine(
    newRatingMatrix,
    userVectors, itemVectors,
    userBiases, itemBiases,
    regularization, gamma,
    iterations, factors
  );

  Matrix X = engine.train();

  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void trainLogisticMF(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  double regularization = args[1]->NumberValue();
  double gamma = args[2]->NumberValue();
  size_t factors = args[3]->IntegerValue();
  size_t iterations = args[4]->IntegerValue();
  size_t seed = args[5]->IntegerValue();

  size_t numUsers = ratingMatrix.rows();
  size_t numItems = ratingMatrix.cols();
  size_t numZeros = numUsers * numItems;
  double total = 0;

  for(size_t i = 0; i < numUsers; i++) {
    for(size_t j = 0; j < numItems; j++) {
      double value = ratingMatrix(i, j);
      if(value == 0) continue;
      numZeros -= 1;
      total += value;
    }
  }

  double alpha = numZeros / total;
  Matrix newRatingMatrix = ratingMatrix * alpha;

  LogisticMFEngine engine(
    newRatingMatrix,
    factors, iterations,
    regularization, gamma,
    seed
  );

  Matrix X = engine.train();

  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void testALS(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  Matrix userFactors(Local<Array>::Cast(args[1]));
  Matrix itemFactors(Local<Array>::Cast(args[2]));

  double regularization = args[3]->NumberValue();
  size_t iterations = args[4]->IntegerValue();

  ALSEngine engine(
    ratingMatrix,
    userFactors,
    itemFactors,
    regularization,
    iterations
  );

  Matrix X = engine.train();
  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void trainALS(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  double alpha = args[1]->NumberValue();
  size_t factors = args[2]->IntegerValue();
  double regularization = args[3]->NumberValue();
  size_t iterations = args[4]->IntegerValue();
  size_t seed = args[5]->IntegerValue();

  ALSEngine engine(
    ratingMatrix,
    alpha,
    regularization,
    factors,
    iterations,
    seed
  );

  Matrix X = engine.train();
  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void auc(const FunctionCallbackInfo<Value>& args) {

  Local<Array> labels = Local<Array>::Cast(args[0]);
  Local<Array> scores = Local<Array>::Cast(args[1]);

  size_t length = labels->Length();

  std::vector<double> newLabels;
  std::vector<double> newScores;

  for(size_t i = 0; i < length; i++) {
    newLabels.push_back( labels->Get(i)->NumberValue() );
    newScores.push_back( scores->Get(i)->NumberValue() );
  }

  AUC aucMetric;
  double aucScore = aucMetric.compute(newLabels, newScores);

  args.GetReturnValue().Set(aucScore);
}

void trainBPR(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Matrix ratingMatrix(Local<Array>::Cast(args[0]));
  size_t factors = args[1]->IntegerValue();
  size_t iterations = args[2]->IntegerValue();
  size_t seed = args[3]->IntegerValue();
  double userLambda = args[4]->NumberValue();
  double itemLambda = args[5]->NumberValue();
  double learningRate = args[6]->NumberValue();
  size_t numNegativeSamples = args[7]->IntegerValue();
  size_t evalNumNeg = args[8]->IntegerValue();
  int32_t evalSeed = args[9]->IntegerValue();
  double decayRate = args[10]->NumberValue();
  bool shuffleTrainingSet = args[11]->BooleanValue();
  double initDistributionBound = args[12]->NumberValue();

  BPREngine engine(
    ratingMatrix,
    factors, iterations, seed,
    userLambda, itemLambda,
    learningRate,
    numNegativeSamples,
    evalNumNeg, evalSeed,
    decayRate,
    shuffleTrainingSet,
    initDistributionBound
  );

  Matrix X = engine.optimize();
  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

double mean( std::vector<double>& vect ){
  double total = 0;
  for( size_t i = 0; i < vect.size(); i++ ){
    total += vect[i];
  }

  return total / vect.size();
}

void bm25(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = args.GetIsolate();

  Local<Array> dataset = Local<Array>::Cast(args[0]);
  Matrix X(dataset);

  size_t N = X.rows();
  double K1 = args[1]->NumberValue();
  double B = args[2]->NumberValue();

  std::vector<size_t> xCol;
  for(size_t i = 0; i < N; i++ ){
    for(size_t j = 0; j < X.cols(); j++ ){
      if( X(i, j) > 0 ){
        xCol.push_back(j);
      }
    }
  }

  std::vector<double> idf(X.cols());
  for( size_t i = 0; i < idf.size(); i++ ){
    int num = 0;
    for( size_t j = 0; j < xCol.size(); j++ ){
      if( xCol[j] == i ){
        num += 1;
      }
    }
    idf[i] = log( double(N) / (num + 1) );
  }

  std::vector<double> rowSums(N);
  for( size_t i = 0; i < N; i++ ){
    Vector vect = X(i);
    rowSums[i] = vect.sum();
  }

  double averageLength = mean(rowSums);

  std::vector<double> lengthNorm(N);
  for( size_t i = 0; i < N; i++ ){
    lengthNorm[i] = (1.0 - B) + B *  rowSums[i] / averageLength;
  }

  for( size_t i = 0; i < N; i++ ){
    for( size_t j = 0; j < X.cols(); j++ ){
      X(i,j) = X(i,j) * (K1 + 1.0) / (K1 * lengthNorm[i] + X(i,j)) * idf[j];
      if( std::isnan( X(i,j) ) ){
        X(i, j) = 0;
      }
    }
  }

  args.GetReturnValue().Set(X.convertToLocalArray(isolate));
}

void init(Local<Object> exports) {

  // Weighting
  NODE_SET_METHOD(exports, "bm25", bm25);

  // Metrics
  NODE_SET_METHOD(exports, "auc", auc);

  // Production Algorithms
  NODE_SET_METHOD(exports, "trainLogisticMF", trainLogisticMF);
  NODE_SET_METHOD(exports, "trainALS", trainALS);
  NODE_SET_METHOD(exports, "trainBPR", trainBPR);

  // Testing Algorithms
  NODE_SET_METHOD(exports, "testLogisticMf", testLogisticMf);
  NODE_SET_METHOD(exports, "testALS", testALS);
}

NODE_MODULE(main, init)

}  // namespace demo
